import 'dart:async';
import 'dart:io';

import 'package:flutter/services.dart';

/// AudioManager Streams control the type of volume the getVol and setVol function will control.
enum AudioManager {
  /// Controls the Voice Call volume
  streamVoiceCall,

  /// Controls the system volume
  streamSystem,

  /// Controls the ringer volume
  streamRing,

  /// Controls the media volume
  streamMusic,

  /// Controls the alarm volume
  streamAlarm,

  /// Controls the notification volume
  streamNotification
}

/// You can control VoiceCall, System, Ringer, Media, Alarm, Notification
/// volume and get the max possible volumes for the respective.
///
/// Call the initAudioStream(AudioManager audioManager) function in initState()
///
/// To make sure the setVol({int androidVol = 0, double iOSVol = 0.0, bool showVolumeUI = true}), getMaxVol, and getVol control
///
/// The volume passed as the parameter to the initAudioStream(AudioManager audioManager) function.
class Volume {
  /// Flutter method channel
  static const MethodChannel _channel = MethodChannel('lecle_volume_flutter');

  /// Pass any AudioManager Stream as a parameter to this function and the
  /// volume buttons and setVol (...) function will control that particular volume.
  ///
  /// When you change the audio manager like in the example app, you should call this function to reinit the audio stream.
  static Future<void> initAudioStream(AudioManager audioManager) async {
    Map<String, int?> map = <String, int?>{};
    map.putIfAbsent("streamType", () {
      return _getStreamInt(audioManager);
    });
    await _channel.invokeMethod('initVolumeStream', map);
  }

  /// Returns an int which is the Max Possible volume for the selected
  /// AudioManager Stream same as that passed in the
  /// controlVolume ( AudioManager )  function.
  ///
  /// Can be implemented like this :-
  ///
  /// int maxVol = await Volume.getMaxVol;
  static Future<dynamic> get getMaxVol async {
    var maxVol = await _channel.invokeMethod('getMaxVol');
    return maxVol;
  }

  /// Returns an int which is the current volume for the selected
  /// AudioManager Stream same as that passed in the
  /// controlVolume ( AudioManager )  function.
  ///
  /// Can be implemented like this :-
  ///
  /// int currentVol = await Volume.getVol;
  static Future<dynamic> get getVol async {
    var vol = await _channel.invokeMethod('getVol');
    return vol;
  }

  /// Call this function with an integer value to set the volume of the selected.
  /// OPTIONAL PARAMETER [ShowVolumeUI showVolumeUI]
  /// to show or hide system Volume UI while changing the volume.
  /// AudioManager stream but value should be less then value returned by getMaxVol getter
  ///
  /// Can be implemented as :-
  ///
  /// await Volume.setVol ( int i, ShowVolumeUI showVolumeUI );
  ///
  /// where value of 'i' is less then Volume.getMaxVol
  ///
  /// value of showVolumeUI can have two values [ShowVolumeUI.show] and [ShowVolumeUI.hide]
  static Future<int> setVol({
    int androidVol = 0,
    double iOSVol = 0.0,
    bool showVolumeUI = true,
  }) async {
    Map<String, dynamic> map = <String, dynamic>{};

    map.putIfAbsent("newVol", () {
      return Platform.isAndroid
          ? androidVol
          : Platform.isIOS
              ? iOSVol
              : 0.0;
    });
    map.putIfAbsent("showVolumeUiFlag", () {
      return _getShowVolumeUiInt(showVolumeUI);
    });
    int vol = await _channel.invokeMethod('setVol', map);
    return vol;
  }

  /// Press VolumeUp button programmatically.
  /// It returns a null.
  ///
  /// Implementation :-
  ///
  /// Volume.volUp()
// static Future<Null> volUp() async{
//   await SystemShortcuts.volUp();
// }

  /// Press VolumeDown button programmatically.
  /// It returns a null.
  ///
  /// Implementation :-
  ///
  /// Volume.volDown()
// static Future<Null> volDown() async{
//   await SystemShortcuts.volDown();
// }
}

/// Get the int value for invokeMethod to show the volume slider UI or not
int _getShowVolumeUiInt(bool showVolumeUI) {
  if (showVolumeUI) {
    return 1;
  }

  return 0;
}

/// Get the stream type int value for invokeChannel
int? _getStreamInt(AudioManager audioManager) {
  switch (audioManager) {
    case AudioManager.streamVoiceCall:
      return 0;
    case AudioManager.streamSystem:
      return 1;
    case AudioManager.streamRing:
      return 2;
    case AudioManager.streamMusic:
      return 3;
    case AudioManager.streamAlarm:
      return 4;
    case AudioManager.streamNotification:
      return 5;
    default:
      return null;
  }
}
